#
#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Cambodia levels (also known as grades).
"""

from persistent import Persistent

from zope.interface import implements, implementer
from zope.component import adapts, adapter
from zope.container.interfaces import INameChooser
from zope.container.btree import BTreeContainer
from zope.container.ordered import OrderedContainer
from zope.container.contained import Contained
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.intid import addIntIdSubscriber
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent.interfaces import IObjectAddedEvent
from zope.lifecycleevent.interfaces import IObjectRemovedEvent
from zope.lifecycleevent import ObjectAddedEvent
from zope.component import getUtility

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.course.interfaces import ICourseContainer
from schooltool.course.course import Course
from schooltool.schoolyear.interfaces import ISchoolYearContainer, ISchoolYear
from schooltool.schoolyear.subscriber import ObjectEventAdapterSubscriber
from schooltool.relationship import URIObject, RelationshipSchema
from schooltool.relationship import RelationshipProperty
from schooltool.app.app import InitBase, StartUpBase
from schooltool.cambodia.grade import interfaces


URILevelCourses = URIObject(
    'http://schooltool.org/ns/cambodia/levelcourses/',
    'Courses available for the level.',
    'Courses mandated (available to teach) by minsitry.')

URICourse = URIObject(
    'http://schooltool.org/ns/cambodia/levelcourses/course',
    'Course',
    'The course to teach.')

URILevel = URIObject(
    'http://schooltool.org/ns/cambodia/levelcourses/level',
    'Level',
    'The level itself.')


LevelCourses = RelationshipSchema(URILevelCourses,
                                 level=URILevel,
                                 course=URICourse)


LEVELS_APP_KEY = 'schooltool.cambodia.grade.level'


class LevelContainerContainer(BTreeContainer):
    """Container of level containers."""

    implements(interfaces.ILevelContainerContainer,
               IAttributeAnnotatable)


class LevelContainer(OrderedContainer):
    """Container of levels."""

    implements(interfaces.ILevelContainer, IAttributeAnnotatable)


class VivifyLevelContainerContainer(object):

    def __call__(self):
        if LEVELS_APP_KEY not in self.app:
            self.app[LEVELS_APP_KEY] = LevelContainerContainer()


class LevelInit(VivifyLevelContainerContainer, InitBase):
    pass


class LevelStartUp(VivifyLevelContainerContainer, StartUpBase):
    pass


@adapter(ISchoolYear)
@implementer(interfaces.ILevelContainer)
def getLevelContainer(sy):
    # XXX: oh really?
    addIntIdSubscriber(sy, ObjectAddedEvent(sy))
    int_ids = getUtility(IIntIds)
    sy_id = str(int_ids.getId(sy))
    app = ISchoolToolApplication(None)
    lc = app[LEVELS_APP_KEY].get(sy_id, None)
    if lc is None:
        lc = app[LEVELS_APP_KEY][sy_id] = LevelContainer()
    return lc


@adapter(ISchoolToolApplication)
@implementer(interfaces.ILevelContainer)
def getLevelContainerForApp(app):
    syc = ISchoolYearContainer(app)
    sy = syc.getActiveSchoolYear()
    if sy is None:
        return None
    return interfaces.ILevelContainer(sy)


@adapter(interfaces.ILevelContainer)
@implementer(ISchoolYear)
def getSchoolYearForLevelContainer(level_container):
    container_id = int(level_container.__name__)
    int_ids = getUtility(IIntIds)
    container = int_ids.getObject(container_id)
    return container


@adapter(interfaces.ILevel)
@implementer(ISchoolYear)
def getSchoolYearForLevel(level):
    level_container = level.__parent__
    return ISchoolYear(level_container)


class Level(Persistent, Contained):

    implements(interfaces.ILevel, interfaces.ILevelContained,
               IAttributeAnnotatable)
    courses = RelationshipProperty(URILevelCourses,
                                   URILevel,
                                   URICourse)

    def __init__(self, title=None):
        self.title = title


DEFAULT_COURSES = (
    {'grades': (1, 6),
     'courses': (
         (u'Mathematic', 10),
         (u'Science & Social Studies', 10),
         (u'Geography', 10),
         (u'History', 10),
         )},
    {'grades': (7, 8),
     'courses': (
         (u'Khmer', 100),
         (u'Mathematics', 100),
         (u'Physics', 50),
         (u'Chimistry', 50),
         (u'Biology', 50),
         (u'Earth and Environmental Studies', 50),
         (u'Morals/Civics', 50),
         (u'Geography', 50),
         (u'History', 50),
         (u'Forieng Languages', 50),
         (u'Sport', 50),
         (u'Art', 50),
         )},
    {'grades': (9, 9),
     'courses': (
         (u'Khmer', 100),
         (u'Mathematics', 100),
         (u'Physics', 30),
         (u'Chimistry', 28),
         (u'Biology', 28),
         (u'Earth and Environmental Studies', 28),
         (u'Morals/Civics', 30),
         (u'Geography', 28),
         (u'History', 28),
         (u'Foreign Languages', 50),
         )},
    {'grades': (10, 10),
     'courses': (
         (u'Khmer', 150),
         (u'Mathematics', 150),
         (u'Physics', 50),
         (u'Chimistry', 37),
         (u'Biology', 38),
         (u'Earth and Environmental Studies', 25),
         (u'History', 37),
         (u'Geography', 38),
         (u'Morals/Civics', 38),
         (u'Forieng Languages', 100),
         (u'Sport and Health Education', 50),
         (u'Life Skill', 50),
         )},
    {'grades': (11, 11),
     'courses': (
         (u'Khmer', 150),
         (u'Foreign Languages', 100),
         (u'Sport and Health Education', 50),
         (u'Mathematics', 100),
         (u'Physics', 100),
         (u'Chimistry', 100),
         (u'Biology', 100),
         (u'Earth and Environmental Studies', 100),
         (u'Geography', 100),
         (u'History', 100),
         (u'Morals/Civics', 100),
         (u'ICT', 100),
         )},
    {'grades': (12, 12),
     'courses': (
         (u'Khmer', 100),
         (u'Mathematics', 125),
         (u'Physics', 75),
         (u'Chimistry', 50),
         (u'Biology', 50),
         (u'Earth and Environmental Studies', 25),
         (u'History', 37),
         (u'Geography', 38),
         (u'Morals/Civics', 50),
         (u'Forieng Languages', 50),
     )},
    )


class InitLevelsForNewSchoolYear(ObjectEventAdapterSubscriber):

    adapts(IObjectAddedEvent, ISchoolYear)

    def makeCourse(self, level, course_title, course_credits):
        schoolyear = ISchoolYear(level)
        courses = ICourseContainer(schoolyear)
        namechooser = INameChooser(courses)

        course = Course(title=course_title, description=u'')
        course.credits = course_credits
        # XXX: I'm pretty sure our namechooser won't handle Khmer
        name = namechooser.chooseName('', course)
        name = namechooser.chooseName(
            '%s-%s' % (level.__name__, name), course)
        courses[name] = course
        return course

    def initDefaultLevels(self, level_container, course_defaults):
        for grade_batch in course_defaults:
            min_grade, max_grade = grade_batch['grades']
            for n in range(min_grade, max_grade+1):
                name = '%d' % n
                if name in level_container:
                    # do not overwrite existing level courses
                    continue
                level = level_container[name] = Level(name)
                for title, credits in grade_batch['courses']:
                    course = self.makeCourse(level, title, credits)
                    LevelCourses(level=level, course=course)

    def copyAllLevels(self, source, destination):
        # TODO: copy levels from previous year?
        # TODO: copy courses to corresponding levels?
        # TODO: anything else?
        return

    def __call__(self):
        app = ISchoolToolApplication(None)
        syc = ISchoolYearContainer(app)
        active_schoolyear = syc.getActiveSchoolYear()

        if active_schoolyear:
            self.copyAllLevels(interfaces.ILevelContainer(active_schoolyear),
                               interfaces.ILevelContainer(self.object))
        else:
            levels = interfaces.ILevelContainer(self.object)
            global DEFAULT_COURSES
            self.initDefaultLevels(levels, DEFAULT_COURSES)


class RemoveLevelsWhenSchoolYearIsDeleted(ObjectEventAdapterSubscriber):
    adapts(IObjectRemovedEvent, ISchoolYear)

    def __call__(self):
        level_container = ILevelContainer(self.object)
        for level_id, level in list(level_container.items()):
            del level_container[level_id]


