#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Gradebook views for schooltool.cambodia
"""

from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility
from zope.security.proxy import removeSecurityProxy
from zope.traversing.browser.absoluteurl import absoluteURL

from schooltool.app.browser.report import ReportPDFView
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.course.interfaces import ISection, ILearner
from schooltool.gradebook.activity import Worksheet, Activity
from schooltool.gradebook.interfaces import IActivities, IGradebook
from schooltool.requirement.interfaces import IEvaluations
from schooltool.requirement.scoresystem import RangedValuesScoreSystem
from schooltool.requirement.scoresystem import UNSCORED
from schooltool.term.interfaces import ITerm, IDateManager
from schooltool.schoolyear.interfaces import ISchoolYear

from schooltool.cambodia import CambodiaMessage as _


class BasePDFView(ReportPDFView):
    """A base class for all PDF views"""

    def __init__(self, context, request):
        super(BasePDFView, self).__init__(context, request)
        self.current_term = getUtility(IDateManager).current_term
        if self.current_term is None:
            self.schoolyear = None
        else:
            self.schoolyear = ISchoolYear(self.current_term)
        if 'term' in self.request:
            self.term = self.schoolyear[self.request['term']]
        else:
            self.term = None

    def noCurrentTerm(self):
        if self.current_term is None:
            next_url = absoluteURL(ISchoolToolApplication(None), self.request)
            next_url += '/no_current_term.html'
            self.request.response.redirect(next_url)
            return True
        return False


class BaseStudentPDFView(BasePDFView):
    """A base class for all student PDF views"""

    def getAverage(self, worksheet, student):
        gb = removeSecurityProxy(IGradebook(worksheet))
        total, average = gb.getWorksheetTotalAverage(worksheet, student)
        if average is UNSCORED:
            return ''
        return unicode(average)

    def getGrid(self, student, sections):
        evaluations = IEvaluations(student)
        if not sections:
            return {
                'headings': [],
                'widths': '6cm',
                'rows': [],
                }

        else:
            rows = []
            for section in sections:
                grid_scores = []
                for worksheet in IActivities(section).values():
                    grid_scores.append(self.getAverage(worksheet, student))

                row = {
                    'title': list(section.courses)[0].title,
                    'scores': grid_scores,
                    }
                rows.append(row)

            return {
                'headings': [_('Semester 1'), _('Semester 2'), _('Final')],
                'widths': '6cm,3cm,3cm,3cm',
                'rows': rows,
                }


class BaseReportCardPDFView(BaseStudentPDFView):
    """The report card (PDF) base class"""

    template=ViewPageTemplateFile('report_card_rml.pt')

    def __call__(self):
        """Make sure there is a current term."""
        if self.noCurrentTerm():
            return
        return super(BaseReportCardPDFView, self).__call__()

    def title(self):
        return _('Report Card') + ': ' + self.schoolyear.title

    def course_heading(self):
        return _('Courses')

    def students(self):
        results = []
        for student in self.collectStudents():
            student_name = u'%s %s' % (
                student.first_name, student.last_name)
            student_title = _('Student') + ': ' + student_name

            sections = []
            for section in ILearner(student).sections():
                term = ITerm(section)
                schoolyear = ISchoolYear(term)
                if schoolyear != self.schoolyear:
                    continue
                if self.term and term != self.term:
                    continue
                sections.append(section)

            result = {
                'title': student_title,
                'grid': self.getGrid(student, sections),
                }

            results.append(result)
        return results


class StudentReportCardPDFView(BaseReportCardPDFView):
    """A view for printing a report card for a student"""

    def collectStudents(self):
        return [self.context]


class GroupReportCardPDFView(BaseReportCardPDFView):
    """A view for printing a report card for each person in a group"""

    def collectStudents(self):
        return list(self.context.members)

