#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Cambodia person browser views.
"""
from zope.app.pagetemplate import ViewPageTemplateFile
from zope.catalog.interfaces import ICatalog
from zope.component import getUtility, adapts
from zope.container.interfaces import INameChooser
from zope.intid.interfaces import IIntIds
from zope.traversing.browser.absoluteurl import absoluteURL
from z3c.form import form, field, button
from zc.table import table
from zope.schema import TextLine, getFieldNamesInOrder
from zope.interface import implements

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.basicperson.browser.person import IPersonAddForm
from schooltool.basicperson.browser.person import PersonAddView
from schooltool.basicperson.browser.person import PersonEditView
from schooltool.group.interfaces import IGroupContainer
from schooltool.person.browser.person import PersonFilterWidget
from schooltool.person.browser.person import PersonContainerView
from schooltool.person.browser.person import PersonTableFormatter
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.term.interfaces import IDateManager
from schooltool.table.batch import Batch
from schooltool.skin.containers import ContainerDeleteView
from schooltool.contact.interfaces import IContact, IContactPersonInfo
from schooltool.contact.interfaces import IContactable, IContactContainer
from schooltool.contact.contact import Contact, ContactPersonInfo

from schooltool.cambodia import CambodiaMessage as _


class IOptionalContact(IContact):

    first_name = TextLine(title=_(u"First name"), required=False)

    last_name = TextLine(title=_(u"Last name"), required=False)


class OptionalContactFormAdapter(object):

    implements(IOptionalContact)
    adapts(IContact)

    def __init__(self, context):
        self.__dict__['context'] = context

    def __setattr__(self, name, value):
        setattr(self.context, name, value)

    def __getattr__(self, name):
        return getattr(self.context, name)


class CambodiaPersonFilterWidget(PersonFilterWidget):

    template = ViewPageTemplateFile('templates/person_filter.pt')

    def filter(self, items):
        if 'CLEAR_SEARCH' in self.request:
            self.request.form['SEARCH_TITLE'] = ''
        # XXX: should we use groups for the current term?
        group = self.groupContainer().get(self.request.form.get('SEARCH_GROUP'))
        if not group:
            return []
        int_ids = getUtility(IIntIds)
        keys = set([int_ids.queryId(person)
                    for person in group.members])
        items = [item for item in items
                 if item['id'] in keys]
        catalog = ICatalog(self.context)
        title_index = catalog['title']
        username_index = catalog['__name__']
        if 'SEARCH_TITLE' in self.request:
            searchstr = self.request['SEARCH_TITLE'].lower()
            results = []
            for item in items:
                title = title_index.documents_to_values[item['id']]
                username = username_index.documents_to_values[item['id']]
                if (searchstr in title.lower() or
                    searchstr in username.lower()):
                    results.append(item)
            items = results
        return items


class CambodiaPersonTableFormatter(PersonTableFormatter):

    def setUp(self, items=None, ommit=[], filter=None, columns=None,
              columns_before=[], columns_after=[], sort_on=None, prefix="",
              formatters=[], table_formatter=table.FormFullFormatter,
              batch_size=25):
        self.filter_widget = CambodiaPersonFilterWidget(self.context, self.request)
        self._table_formatter = table_formatter
        if not columns:
            columns = self.columns()
        if formatters:
            for formatter, column in zip(formatters, columns):
                column.cell_formatter = formatter
        self._columns = columns_before[:] + columns[:] + columns_after[:]
        if items is None:
            items = self.items()
        if not filter:
            filter = self.filter
        self._items = filter(self.ommit(items, ommit))
        self.prefix = prefix
        if batch_size == 0:
            batch_size = len(list(self._items))
        self.batch = Batch(self, batch_size=batch_size)
        self._sort_on = sort_on or self.sortOn()


class PersonsView(PersonContainerView):

    template = ViewPageTemplateFile('templates/person_container.pt')
    delete_template = ViewPageTemplateFile('templates/person_delete.pt')

    group_id = None

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.table = CambodiaPersonTableFormatter(self.context, self.request)
        self.request.form['SEARCH_GROUP'] = self.group_id


class TeachersView(PersonsView):

    index_title = _('Teacher index')
    group_id = 'teachers'


class StudentsView(PersonsView):

    index_title = _('Student index')
    group_id = 'students'


class AdministratorsView(PersonsView):

    index_title = _('Administrator index')
    group_id = 'administrators'


class CambodiaPersonAddView(PersonAddView):

    group_id = None

    def groupViewURL(self):
        return "%s/%s.html" % (absoluteURL(self.context, self.request),
                               self.group_id)

    def groupContainer(self):
        app = ISchoolToolApplication(None)
        return IGroupContainer(app, {})

    # XXX: should we use groups for the current term?
    def currentGroupContainer(self):
        term = getUtility(IDateManager).current_term
        school_year = ISchoolYear(term, None)
        return IGroupContainer(school_year, {})

    @button.buttonAndHandler(_('Add'), name='add')
    def handleAdd(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        if self.fields.get('password') is not None and \
           data['password'] != data['confirm']:
            self.status = _(u'Passwords do not match')
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            # mark only as finished if we get the new object
            self._finishedAdd = True

    @button.buttonAndHandler(_('Cancel'), name='cancel')
    def handle_cancel_action(self, action):
        self.request.response.redirect(self.groupViewURL())

    def create(self, data):
        username = data.get('username')
        first_name = data.get('first_name')
        last_name = data.get('last_name')
        person = self._factory(username, first_name, last_name)
        data.pop('confirm')
        form.applyChanges(self, person, data)
        groups = self.groupContainer()
        group = groups.get(self.group_id)
        if group is not None:
            person.groups.add(group)
        self._person = person
        return person

    def update(self):
        self.fields = field.Fields(IPersonAddForm).omit('group', 'advisor')
        self.updateWidgets()
        self.updateActions()
        self.actions.execute()


class TeacherAddView(CambodiaPersonAddView):

    label = _('Add new teacher')
    group_id = 'teachers'


class StudentAddView(CambodiaPersonAddView):

    label = _('Add new student')
    group_id = 'students'

    # XXX: use subforms for IContact and IContactPersonInfo
    def create(self, data):
        username = data.get('username')
        first_name = data.get('first_name')
        last_name = data.get('last_name')
        person = self._factory(username, first_name, last_name)
        name = INameChooser(self.context).chooseName('', person)
        person.username = name
        person_data = {}
        for key in data:
            if not key.startswith('contact.') and not key.startswith('contact_info'):
                person_data[key] = data[key]
        form.applyChanges(self, person, person_data)
        contact_data = {}
        for key in data:
            if key.startswith('contact.'):
                contact_data[key] = data[key]
        self._contact = Contact()
        form.applyChanges(self, self._contact, contact_data)

        contact_info_data = {}
        for key in data:
            if key.startswith('contact_info.'):
                contact_info_data[key] = data[key]
        self._contact_info = ContactPersonInfo()
        form.applyChanges(self, self._contact_info, contact_info_data)
        groups = self.groupContainer()
        group = groups.get(self.group_id)
        if group is not None:
            person.groups.add(group)
        self._person = person
        return person

    def add(self, person):
        name = person.username
        self.context[name] = person
        contact_container = IContactContainer(ISchoolToolApplication(None))
        name = INameChooser(contact_container).chooseName('', self._contact)
        contact_container[name] = self._contact
        self._contact_info.__parent__ = person
        IContactable(person).contacts.add(self._contact, self._contact_info)
        return person

    def update(self):
        self.fields = field.Fields(IPersonAddForm).omit('group', 'advisor',
                                                        'username', 'password',
                                                        'confirm')
        for field_name in getFieldNamesInOrder(IContact):
            self.fields += field.Fields(IOptionalContact[field_name], prefix='contact')
        self.fields += field.Fields(IContactPersonInfo, prefix='contact_info')
        self.updateWidgets()
        self.updateActions()
        self.actions.execute()


class AdministratorAddView(CambodiaPersonAddView):

    label = _('Add new administrator')
    group_id = 'administrators'


class CambodiaPersonEditView(PersonEditView):

    form.extends(PersonEditView)
    select_fields = ['prefix', 'first_name', 'middle_name', 'last_name',
                     'suffix', 'preferred_name', 'gender', 'birth_date']

    def update(self):
        self.fields = field.Fields(IPersonAddForm).select(*self.select_fields)
        self.updateWidgets()
        self.updateActions()
        self.actions.execute()


class PersonContainerDeleteView(ContainerDeleteView):

    def nextURL(self):
        url = absoluteURL(self.container, self.request)
        if 'nextURL' in self.request:
            url += '/' + self.request['nextURL']
        return url
